import React, { forwardRef, Ref, useImperativeHandle, useState } from 'react'
import { useTranslation } from 'react-i18next'

import {
  Box,
  Grid,
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableRow,
  Typography,
} from '@mui/material'

import format from 'date-fns/format'
import { RegisterFormData } from './formData'
import { postJsonOpts } from '../../utils'
import { PersonFormMethods } from './personFormMethods'
import useOus from '../../hooks/useOus'
import useRoleTypes from '../../hooks/useRoleTypes'
import SubmitState from './submitState'


interface StepSummaryProperties {
  formData: RegisterFormData
  onSubmitSuccess: () => void
  onSubmitFailure: () => void
}

const StepSummary = forwardRef((props: StepSummaryProperties, ref: Ref<PersonFormMethods>) => {
    const { t } = useTranslation(['common'])
    const { formData, onSubmitSuccess, onSubmitFailure } = props
    const ous = useOus()
    const roleTypes = useRoleTypes()
    const [submitState, setSubmitState] = useState(SubmitState.NotSubmitted)

    const submit = () => {
      const payload = {
        first_name: formData.first_name,
        last_name: formData.last_name,
        email: formData.email,
        role: {
          type: formData.role_type,
          start_date: formData.role_start === null ? null : format(formData.role_start as Date, 'yyyy-MM-dd'),
          end_date: formData.role_end === null ? null : format(formData.role_end as Date, 'yyyy-MM-dd'),
          comments: formData.comment,
          orgunit_id: formData.ou_id,
        },
      }

      console.log('submitting', JSON.stringify(payload))
      fetch('/api/ui/v1/invite/', postJsonOpts(payload))
        .then((res) => {
          if (!res.ok) {
            onSubmitFailure()
            setSubmitState(SubmitState.SubmitFailure)
            return null
          }
          return res.text()
        })
        .then((result) => {
          if (result !== null) {
            console.log('result', result)
            onSubmitSuccess()
            setSubmitState(SubmitState.SubmitSuccess)
          }
        })
        .catch((error) => {
          console.log('error', error)
          onSubmitFailure()
          setSubmitState(SubmitState.SubmitFailure)
        })
    }

    function doSubmit() {
      return submit()
    }

    useImperativeHandle(ref, () => ({ doSubmit }))

    return (
      <>
        <Typography variant='h5' sx={{
          paddingTop: '1rem',
          paddingBottom: '1rem',
        }}>{t('registerWizardText.summaryContactInformation')}</Typography>
        <Box sx={{ typography: 'subtitle2' }}>
          {t('registerWizardText.summaryPage')}
        </Box>
        <Box sx={{ maxWidth: '30rem' }}>

          <Box sx={{ padding: '1rem', bgcolor: 'primary.dark' }}>{t('registerWizardText.contactInformation')}</Box>

          <Grid container>
            <Grid item xs={4}>
              {t('input.fullName')}
            </Grid>
            <Grid item xs={8}>
              {formData.first_name} {formData.last_name}
            </Grid>
            <Grid item xs={4}>
              {t('input.email')}
            </Grid>
            <Grid item xs={8}>
              {formData.email}
            </Grid>
          </Grid>

          <Typography variant='h5' sx={{
            paddingTop: '1rem',
            paddingBottom: '1rem',
          }}>{t('registerWizardText.summaryPeriod')}</Typography>

          <Table>
            <TableHead sx={{
              padding: '1rem',
              bgcolor: 'primary.dark',
            }}>

              <TableRow>
                <TableCell>Guest role</TableCell>
                <TableCell>Period</TableCell>
                <TableCell>Department</TableCell>
                <TableCell>Comment</TableCell>
              </TableRow>
            </TableHead>
            <TableBody>
              <TableRow>
                <TableCell>{formData.role_type}</TableCell>
                <TableCell>{formData.role_start === null ? null : format(formData.role_start as Date, 'yyyy-MM-dd')}-{formData.role_end === null ? null : format(formData.role_end as Date, 'yyyy-MM-dd')}</TableCell>
                <TableCell>{formData.ou_id}</TableCell>
                <TableCell>{formData.comment}</TableCell>
              </TableRow>
            </TableBody>
          </Table>

          {/* TODO For now just showing a heading to give the user some feedback */}
          {submitState === SubmitState.SubmitSuccess && (
            <Box>
              <h2>Submit success</h2>
            </Box>
          )}

          {submitState === SubmitState.SubmitFailure && (
            <Box>
              <h2>Submit failure</h2>
            </Box>
          )}
        </Box>
      </>
    )
  },
)

export default StepSummary
